import connexion
from flask import Response
from txtapi.handlers import controller_handler
from txtapi.models.servomotor import Servomotor
from txtapi.models.input import Input  # noqa: E501
from txtapi.models.motor import Motor  # noqa: E501
from txtapi.models.output import Output  # noqa: E501
from txtapi.models.camera_config import CameraConfig  # noqa: E501
from txtapi.models.image_recognition_config import ImageRecognitionConfig  # noqa: E501
from txtapi.models.counter import Counter


def add_input(controller_id, input_id, input_device):  # noqa: E501
    """add_input

    Initializes an input. # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param input_id: Id of the input
    :type input_id: int
    :param input_device: 
    :type input_device: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        input_device = Input.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.create_input(controller_id, input_id, input_device)


def add_counters(controller_id, counters=None):  # noqa: E501
    """add_counters

    Initialize all counters # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param counter:
    :type counter: list | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        counters = [Counter.from_dict(d) for d in connexion.request.get_json()]  # noqa: E501
    return controller_handler.create_counters(controller_id, counters)


def add_inputs(controller_id, input=None):  # noqa: E501
    """add_inputs

    Get values for all inputs # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param input:
    :type input: list | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        input_devices = [Input.from_dict(d) for d in connexion.request.get_json()]  # noqa: E501
    return controller_handler.create_inputs(controller_id, input_devices)


def get_counter(controller_id, counter_id):  # noqa: E501
    """get_counter

    Get value for a counter # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param counter_id: Id of the counter
    :type counter_id: int

    :rtype: Counter
    """
    return controller_handler.get_counter(controller_id, counter_id)


def get_counters(controller_id):  # noqa: E501
    """get_counters

    Get values for all counters # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: List[Counter]
    """
    return controller_handler.get_counters(controller_id)


def get_input(controller_id, input_id):  # noqa: E501
    """get_input

    Get value for an input # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param input_id: Id of the input
    :type input_id: int

    :rtype: Input
    """
    return controller_handler.get_input(controller_id, input_id)


def get_inputs(controller_id):  # noqa: E501
    """get_inputs

    Get values for all inputs # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: List[Input]
    """
    return controller_handler.get_inputs(controller_id)


def init_controller_by_id(controller_id):  # noqa: E501
    """Initialize controller by id

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: None
    """
    return controller_handler.init_controller(controller_id)


def input_message_stream(controller_id, x_api_key=None):  # noqa: E501
    """Event stream of the inputs

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param x_api_key:
    :type x_api_key: str

    :rtype: str
    """
    return Response(controller_handler.input_message_stream(controller_id),
                    mimetype='text/event-stream')


def reset_counter(controller_id, counter_id):  # noqa: E501
    """reset_counter

    Reset all counters # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param counter_id: Id of the counter
    :type counter_id: int

    :rtype: None
    """
    return controller_handler.reset_counter(controller_id, counter_id)


def set_motor(controller_id, motor_id, motor):  # noqa: E501
    """set_motor

    Set speed for specified motor # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param motor_id: Id of the motor
    :type motor_id: int
    :param motor: 
    :type motor: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        motor = Motor.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.set_motor(controller_id, motor_id, motor)


def set_output(controller_id, output_id, output):  # noqa: E501
    """set_output

    Set value for specified output # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param output_id: Id of the output
    :type output_id: int
    :param output: 
    :type output: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        output = Output.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.set_output(controller_id, output_id, output)


def set_position(controller_id, servomotor_id, servomotor):  # noqa: E501
    """set_position

    Set position for specified servomotor # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param servomotor_id: Id of the servomotor
    :type servomotor_id: int
    :param servomotor:
    :type servomotor: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        servomotor = Servomotor.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.set_servomotor(controller_id, servomotor_id, servomotor)


def application_message_stream(x_api_key=None):  # noqa: E501
    """Event stream of the application

     # noqa: E501

    :rtype: str
    """
    return Response(controller_handler.application_message_stream(),
                    mimetype='text/event-stream')


def camera_image_stream(x_api_key=None):  # noqa: E501
    """Start video stream of camera

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: file
    """
    return Response(controller_handler.camera_image_stream(),
                    mimetype='multipart/x-mixed-replace; boundary=frame')


def camera_message_stream(x_api_key=None):  # noqa: E501
    """Start video stream of camera

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: file
    """
    return Response(controller_handler.camera_message_stream(),
                    mimetype='text/event-stream')


def counter_message_stream(controller_id, x_api_key=None):  # noqa: E501
    """Event stream of the counters

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param x_api_key:
    :type x_api_key: str

    :rtype: str
    """
    return Response(controller_handler.counter_message_stream(controller_id),
                    mimetype='text/event-stream')


def discover(x_api_key=None):  # noqa: E501
    """Returns information about controller and controllers connected to it.

     # noqa: E501

    :param x_api_key:
    :type x_api_key: str

    :rtype: List[Controller]
    """
    return controller_handler.discover()


def add_camera_image_recognition_config():  # noqa: E501
    """Start video stream of camera

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param config: Config of the camera stream
    :type config: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        config = ImageRecognitionConfig.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.add_camera_image_recognition_config(config)


def start_camera():  # noqa: E501
    """Start video stream of camera

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int
    :param config: Config of the camera stream
    :type config: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        config = CameraConfig.from_dict(connexion.request.get_json())  # noqa: E501
    return controller_handler.start_camera(config)


def stop_camera():  # noqa: E501
    """Stop video stream of camera

     # noqa: E501

    :param controller_id: Id of the controller
    :type controller_id: int

    :rtype: None
    """
    return controller_handler.stop_camera()


def stop_controller():  # noqa: E501
    """stop_all

    Stop all controller instances. # noqa: E501

    :rtype: None
    """
    return controller_handler.stop_controller()


def start_interface_test():  # noqa: E501
    """Start interface test

    Starts the interface test on the controller. # noqa: E501


    :rtype: None
    """
    return controller_handler.start_interface_test()


def stop_interface_test():  # noqa: E501
    """Stop interface test

    Stops the interface test on the controller. # noqa: E501


    :rtype: None
    """
    return controller_handler.stop_interface_test()