import connexion
from txtapi.handlers import debugger_handler as Handler
from txtapi.models.breakpoint import Breakpoint  # noqa: E501
from txtapi.models.enabled import Enabled  # noqa: E501
from txtapi.models.expression import Expression  # noqa: E501
from txtapi.models.debugger_arguments import DebuggerArguments

def add_breakpoints(workspace_name, breakpoints):  # noqa: E501
    """add_breakpoints

    Adds array off breakpoints with the specified line in the specified file. Returns the same breakpoints with an ID. Might increment the line number if the specified line was blank or a comment. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string 
    :param breakpoints: The breakpoints
    :type breakpoints: list | bytes

    :rtype: List[Breakpoint]
    """
    if connexion.request.is_json:
        breakpoints = [Breakpoint.from_dict(d) for d in connexion.request.get_json()]  # noqa: E501
    return Handler.add_breakpoints(workspace_name, breakpoints)


def add_expressions(workspace_name, expressions):  # noqa: E501
    """add_expressions

    Adds array of variables identified by a key (&#x3D; variable name) that evaluates a command or expression to the watchlist. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string 
    :param expressions: The expressions
    :type expressions: list | bytes

    :rtype: List[Expression]
    """
    if connexion.request.is_json:
        expressions = [Expression.from_dict(d) for d in connexion.request.get_json()]  # noqa: E501
    return Handler.add_expressions(workspace_name, expressions)

def call_continue(workspace_name):  # noqa: E501
    """call_continue

    Send continue command to debugger. The debugger will resume the program until the next breakpoint is hit. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: DebuggerResponse
    """
    return Handler.call_continue(workspace_name)


def callstack_down(workspace_name, steps=None):  # noqa: E501
    """callstack_down

    Move down in the current call stack (to a newer frame). Returns the current frame. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param steps: The number of steps the current frame is moved down in the call stack.
    :type steps: int

    :rtype: DebuggerResponse
    """
    return Handler.callstack_down(workspace_name, steps)


def callstack_up(workspace_name, steps=None):  # noqa: E501
    """callstack_up

    Move up in the current call stack (to an older frame). Returns the current frame. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param steps: The number of steps the current frame is moved up in the call stack.
    :type steps: int

    :rtype: DebuggerResponse
    """
    return Handler.callstack_up(workspace_name, steps)


def delete_all_breakpoints(workspace_name):  # noqa: E501
    """delete_all_breakpoints

    Delete all breakpoints for this workspace_name. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string 

    :rtype: InlineResponseDefault
    """
    return Handler.delete_all_breakpoints(workspace_name)


def delete_breakpoint_by_id(workspace_name, breakpoint_id):  # noqa: E501
    """delete_breakpoint_by_id

    Delete the specified breakpoint. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param breakpoint_id: bp id
    :type breakpoint_id: int

    :rtype: InlineResponseDefault
    """
    return Handler.delete_breakpoint_by_id(workspace_name, breakpoint_id)


def delete_expression_by_key(workspace_name, expression_key):  # noqa: E501
    """delete_expression

    Remove the previously defined expression from the watchlist. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param expression_key: The key of the expression
    :type expression_key: str

    :rtype: InlineResponseDefault
    """
    return Handler.delete_expression_by_key(workspace_name, expression_key)


def get_breakpoints(workspace_name):  # noqa: E501
    """get_breakpoints

    Returns a list of all previously specified breakpoints. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: List[Breakpoint]
    """
    return Handler.get_breakpoints(workspace_name)


def get_callstack(workspace_name):  # noqa: E501
    """get_callstack

    Returns the current call stack of the specified workspace_name. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: List[ProgramLocation]
    """
    return Handler.get_callstack(workspace_name)


def get_expressions(workspace_name):  # noqa: E501
    """get_expressions

    Returns the value of all previously specified expressions. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: List[Expression]
    """
    return Handler.get_expressions(workspace_name)


def get_globals(workspace_name):  # noqa: E501
    """get_globals

    Returns all expressions and their values that are currently defined in the global scope. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: List[Expression]
    """
    return Handler.get_globals(workspace_name)


def get_locals(workspace_name):  # noqa: E501
    """get_locals

    Returns all expressions and their values that are currently defined in the local scope. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: List[Expression]
    """
    return Handler.get_locals(workspace_name)


def restart(workspace_name, debugger_arguments=None):  # noqa: E501
    """restart

    Send restart command to debugger. The debugger will stop and restart the current program. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string 
    :param debugger_arguments: The breakpoints
    :type debugger_arguments: list | bytes

    :rtype: DebuggerResponse
    """
    if connexion.request.is_json:
        debugger_arguments = DebuggerArguments.from_dict(connexion.request.get_json())  # noqa: E501
    return Handler.start(workspace_name, debugger_arguments)


def start(workspace_name, debugger_arguments=None):  # noqa: E501
    """start

    Start the debugger for the specified workspace_name with optional parameters. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string 
    :param debugger_arguments: The breakpoints
    :type debugger_arguments: list | bytes

    :rtype: DebuggerResponse
    """
    if connexion.request.is_json:
        debugger_arguments = DebuggerArguments.from_dict(connexion.request.get_json())  # noqa: E501
    return Handler.start(workspace_name, debugger_arguments)


def step_into(workspace_name):  # noqa: E501
    """step_into

    Send step into command to debugger. The debugger will set the execution pointer at the first statement of the first function called from the current line. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: DebuggerResponse
    """
    return Handler.step_into(workspace_name)


def step_out(workspace_name):  # noqa: E501
    """step_out

    Send step out command to debugger. The debugger will execute the remaining lines of the function by itself and set the execution pointer at the statement following the function call. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: DebuggerResponse
    """
    return Handler.step_out(workspace_name)


def step_over(workspace_name):  # noqa: E501
    """step_over

    Send step over command to debugger. Debugger will step to the next line in the code. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: DebuggerResponse
    """
    return Handler.step_over(workspace_name)


def stop_debugger(workspace_name):  # noqa: E501
    """stop

    Send stop command to debugger. The debugger will stop the execution. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string

    :rtype: InlineResponseDefault
    """
    return Handler.stop_debugger(workspace_name)

def stop_debuggers():  # noqa: E501
    """stop_all

    Stop all running debugger. The debugger will stop the execution. # noqa: E501

    :rtype: None
    """
    return Handler.stop_debuggers()


def toggle_breakpoint(workspace_name, breakpoint_id, enabled):  # noqa: E501
    """toggle_breakpoint

    Update the state of the specified breakpoint and set it to enabled or disabled. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param breakpoint_id: The ID of the breakpoint
    :type breakpoint_id: int
    :param enabled: enabled &#x3D; true sets the breakpoint to enabled, enabled &#x3D; false disables it.
    :type enabled: dict | bytes

    :rtype: Breakpoint
    """
    if connexion.request.is_json:
        enabled = Enabled.from_dict(connexion.request.get_json())  # noqa: E501
    return Handler.toggle_breakpoint(workspace_name, breakpoint_id, enabled)


def toggle_breakpoints(workspace_name, enabled):  # noqa: E501
    """toggle_breakpoints

    Update all breakpoints for this workspace_name and set them to either enabled or disabled. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param enabled: enabled &#x3D; true sets all previously defined breakpoints to enabled, enabled &#x3D; false disables them.
    :type enabled: dict | bytes

    :rtype: List[Breakpoint]
    """
    if connexion.request.is_json:
        enabled = Enabled.from_dict(connexion.request.get_json())  # noqa: E501
    return Handler.toggle_breakpoints(workspace_name, enabled)


def update_expression(workspace_name, expression_key, command):  # noqa: E501
    """update_expression

    Change an expression. # noqa: E501

    :param workspace_name: The workspace name
    :type workspace_name: string
    :param expression_key: The key of the expression
    :type expression_key: str
    :param command: The modified command or expression
    :type command: str

    :rtype: Expression
    """
    return Handler.update_expression(workspace_name, expression_key, command)
